#!/usr/bin/env perl
#!/bin/env perl

## Name: fusionstrandtsc.pl
## Version: 1.2.0
## Date: Jan 01, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# fusion trancripts into non-overlapped transcripts
# --input, input gff, with gene id appearing at the beginning of the 8th field
# --prefix, secondary gene prefix, which should be ignored while overlapping with main gene prefix, e.g. UNS
# --note, print all notes, default: ID only
# --output, output gff
# --form, join only isoforms (highly overlapped transcripts)

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $prefix;
my $note;
my $form;
my $output = "output.gff";
my $line;
my @args;
my @argsold;
my $i;
my @geneid;
my $common;
my $leftcommon;
my $rightcommon;

usage() if (@ARGV < 4 or
	    ! GetOptions("help|?" => \$help,
			 "note" => \$note,
			 "form" => \$form,
			 "input=s" => \$input,
			 "prefix=s" => \$prefix,
			 "output=s" => \$output)
	    or defined $help);

open(IN, $input) or die("Input file not found\n");
open(OUT, ">$output");

$line = <IN>;
if ($line) {
    chomp($line);
    @argsold = split("\t", $line);
}
if ($line && eof) {
    for ($i=0; $i<@argsold-1; $i++) {
	print OUT "$argsold[$i]\t";
    }
    if ($note) {
	print OUT "$argsold[$i]\n";
    }
    else {
	@geneid = split(";", $argsold[$i]);
	print OUT "$geneid[0]\n";
    }
}
while ($line = <IN>) {
    chomp($line);
    @args = split("\t", $line);

    #<while overlapped do fusion
    if ($form) {
	$common = &min($args[4], $argsold[4]) - $args[3];
	$leftcommon = $args[3] - $argsold[3];
	$rightcommon = $args[4] - $argsold[4];
	while ($common > &min($leftcommon, $rightcommon) && $args[0] eq $argsold[0]) {
	    @argsold = &fusion(\@argsold, \@args);
	    $line = <IN>;
	    if ($line) {
		chomp($line);
		@args = split("\t", $line);
	    }
	    else {
		@args = (0,0,0,0,0);
	    }
	    $common = &min($args[4], $argsold[4]) - $args[3];
	    $leftcommon = $args[3] - $argsold[3];
	    $rightcommon = $args[4] - $argsold[4];
       	}
    }
    else {
	while ($args[3] < $argsold[4] && $args[0] eq $argsold[0]) {
	    @argsold = &fusion(\@argsold, \@args);
	    $line = <IN>;
	    if ($line) {
		chomp($line);
		@args = split("\t", $line);
	    }
	    else {
		@args = (0,0,0,0,0);
	    }
	}
    }
    #>

    for ($i=0; $i<@argsold-1; $i++) {
	print OUT "$argsold[$i]\t";
    }
    if ($note) {
	print OUT "$argsold[$i]\n";
    }
    else {
	@geneid = split(";", $argsold[$i]);
	print OUT "$geneid[0]\n";
    }
    @argsold = @args;
    if ($line && eof) {
	for ($i=0; $i<@argsold-1; $i++) {
	    print OUT "$argsold[$i]\t";
	}
	if ($note) {
	    print OUT "$argsold[$i]\n";
	}
	else {
	    @geneid = split(";", $argsold[$i]);
	    print OUT "$geneid[0]\n";
	}
    }
}
close(IN);
close(OUT);
exit 0;

sub max {
    if ($_[0] > $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub min {
    if ($_[0] < $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub fusionstrand {
    if ($_[0] ne ".") {
	$_[0]
    }
    else {
	$_[1]
    }
}

sub fusion {
    my ($a, $b) = @_;
    my @ref = @{$a};
    $ref[3] = min(${$a}[3], ${$b}[3]);	
    $ref[4] = max(${$a}[4], ${$b}[4]);
    $ref[6] = fusionstrand(${$a}[6], ${$b}[6]);

    my @geneida = split(";", ${$a}[8]);
    my @geneidb = split(";", ${$b}[8]);
  
    if (index($geneida[0], $prefix)==-1) {
	$ref[8] = ${$a}[8];
    }
    else {
	if (index($geneidb[0], $prefix)==-1) {
	    $ref[8] = ${$b}[8];
	}
	else {
	    $ref[8] = ${$a}[8];
	}
    }

    return @ref;
}

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl fusionstrandtsc.pl --input GFF --prefix PRE [--form] [--note] [--output OUT] [--help|?] \n";
    exit 1;
}
