#!/usr/bin/env perl
#!/bin/env perl

## Name: makeRNAprof.pl
## Version: 1.2.0
## Date: Jan 01, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Create configuration files for RNAprof execution

use strict;
use warnings;
use Getopt::Long qw(:config no_ignore_case);

my $help;
my $input;
my $cluster;
my $path=".";
my $gffprof="input_";
my $genome="genomesize.txt";
my $node="1";
my $src;
my $memory="2G";
my $output="output";

my $line;
my @args;
my @chromosomes;
my %conditions;
my %order_conds;
my $key1;
my $key2;
my $order;
my $Rsrc;

usage() if (($#ARGV<0) or !(GetOptions (
				'help|?' => \$help,
				'cluster' => \$cluster,
				'input=s' => \$input,
				'gffprof=s' => \$gffprof,
				'GenomeSize=s' => \$genome,
				'memory=s' => \$memory,
				'path=s' => \$path,
				'node=s' => \$node,
				'src=s' => \$src,
				'outputPrefix=s' => \$output
			    ))
	    or defined($help));

$Rsrc = $src."/R";

open(IN, "$input") or die("Whole description file not found\n");
$line = <IN>;
$order = 1;
while ($line=<IN>) {
    chomp($line);
    @args = split("\t", $line);
    push(@{$conditions{$args[2]}}, $line);
    if (!$order_conds{$args[2]}) {
	$order_conds{$args[2]} = $order++;
    }
}
close(IN);

open(CHR, "$genome") or die("Genome size file not found\n");
@chromosomes = ();
while ($line = <CHR>) {
    chomp($line);
    @args = split("\t", $line);
    push(@chromosomes, $args[0]);
}
close(CHR);

foreach $key1 (keys %conditions) {
    foreach $key2 (keys %conditions) {
	if ($order_conds{$key1} < $order_conds{$key2}) { 
	    for (my $i=0; $i<=$#chromosomes; $i++) {
		open(OUT, ">${output}_${key1}_${key2}_$chromosomes[$i].txt");
		print OUT "label\tfile\texperiment\tsize\n";
		foreach (@{$conditions{$key1}}) {
		    @args = split("\t", $_);
		    $args[1] = $path."/curve/".$args[1]."_".$chromosomes[$i].".cur";
		    print OUT join("\t", @args);
		    print OUT "\n";
		}
		foreach (@{$conditions{$key2}}) {
		    @args = split("\t", $_);
		    $args[1] = $path."/curve/".$args[1]."_".$chromosomes[$i].".cur";
		    print OUT join("\t", @args);
		    print OUT "\n";
		}
		close(OUT);
		
		open(OUTPROF, ">${output}_${key1}_${key2}_$chromosomes[$i].sh");
		if ($cluster) {
		    print OUTPROF "#\$ -S /bin/bash\n";
		    print OUTPROF "#\$ -N runRNAprof_${key1}_${key2}_$chromosomes[$i]\n";
		    print OUTPROF "#\$ -V\n";
		    print OUTPROF "#\$ -cwd\n";
		    print OUTPROF "##\$ -q main.q\@node".$node."\n";
		    print OUTPROF "#\$ -pe parallel 1\n";
		    print OUTPROF "#\$ -l h_vmem=".$memory."\n";
		    print OUTPROF "\n";
		}
		else {
		    print OUTPROF "#!/usr/bin/env bash\n";
		    print OUTPROF "#!/bin/env bash\n";
		    print OUTPROF "\n";
		}
		print OUTPROF "DIR=$path\n";
		print OUTPROF "SRC=$Rsrc\n";
		print OUTPROF "GFF=\$DIR/rnaseq/gff/${gffprof}annotation_noexon_$chromosomes[$i].gff\n";
		print OUTPROF "EXON=\$DIR/rnaseq/exon/${gffprof}exon_$chromosomes[$i].tsv\n";
		print OUTPROF "DESC=${output}_${key1}_${key2}_$chromosomes[$i].txt\n";
		print OUTPROF "OUTPUT=\$DIR/result/rnaprof_${key1}_${key2}forpanr_$chromosomes[$i]/\n";
		print OUTPROF "MINLENGTH=\$1\n";
		print OUTPROF "GLOBAL_P=\$2\n";
		print OUTPROF "LOCAL_P=\$3\n\n";
		
		print OUTPROF "if [ ! -d \"\$DIR/result/rnaprof_${key1}_${key2}forpanr_$chromosomes[$i]\" ]\n";
		print OUTPROF "then\n";
		print OUTPROF "mkdir \$DIR/result/rnaprof_${key1}_${key2}forpanr_$chromosomes[$i]\n";
		print OUTPROF "fi\n";
		
		print OUTPROF "Rscript \$SRC/RNAprof.R -i \$DESC -G \$GFF -e \$EXON -f \$OUTPUT -L \$MINLENGTH -P \$GLOBAL_P -p \$LOCAL_P\n";
		close(OUTPROF);
	    }
	}
    }
}

exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: see coverage.sh \n";
    exit 1;
}
